
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "../src/initng.h"

#include <time.h>                           /* time() */
#include <fcntl.h>                          /* fcntl() */
#include <sys/un.h>                         /* memmove() strcmp() */
#include <sys/wait.h>                       /* waitpid() sa */
#include <linux/kd.h>                       /* KDSIGACCEPT */
#include <sys/ioctl.h>                      /* ioctl() */
#include <stdlib.h>                         /* free() exit() */
#include <sys/reboot.h>                     /* reboot() RB_DISABLE_CAD */
#include <termios.h>
#include <stdio.h>
#include <sys/klog.h>
#include <dirent.h>
#include <sys/stat.h>
#include <fnmatch.h>
#include <sys/time.h>
#include <sys/resource.h>

#include "../src/initng_global.h"
#include "../src/main.h"
#include "../src/initng_signal.h"
#include "../src/initng_handler.h"
#include "../src/initng_execute.h"          /* new_environ() */
#include "../src/initng_active_db.h"
#include "../src/initng_load_module.h"
#include "../src/initng_plugin_callers.h"
#include "../src/initng_toolbox.h"
#include "../src/initng_main.h"
#include "../src/initng_service_data_types.h"
#include "../src/initng_fd.h"
#include "../src/initng_common.h"

/* include some soruce files directly, remember this is only a test */
#include "../src/initng_global.c"
#include "../plugins/stcmd/print_service.c"

/*
 * A test, that will load service, set by argument 1, and then exit
 */
static void print_s(char *name);
static void parse_all(const char *dirname);

int main(int argc, char *argv[], char *env[])
{

    /* initialise global variables */
    initng_new(argc, argv, env);

    if (!initng_load_all_modules())
    {
        printf("could not load all modules\n");
        goto exit;
    }

    /* enable verbose mode */
#ifdef DEBUG
    g.verbose = 0;
#endif

    if (argv[1])
    {
        print_s(argv[1]);
    }
    else
    {
        parse_all("system");
        parse_all("daemon");
        parse_all("net");
        parse_all("debug");
        parse_all("daemon/bluetooth");
    }

  exit:
    /* unload all modules */
    initng_unload_all_modules();
    initng_free();
    return (0);
}

static void parse_all(const char *dirname)
{
    struct dirent *e;
    char tmp[200];
    char tmp2[200];
    DIR *d;
    int i;

    sprintf(tmp, "/etc/initng/%s", dirname);

    d = opendir(tmp);
    if (!d)
        return;

    while ((e = readdir(d)))
    {
        if (e->d_name[0] == '.')
            continue;

        /* get string lenght */
        i = strlen(e->d_name);

        /* Check that is it a .i file */
        if (e->d_name[i - 1] != 'i' && e->d_name[i - 2] != '.')
            continue;

        strncpy(tmp, e->d_name, i - 2);
        tmp[i - 2] = '\0';
        sprintf(tmp2, "%s/%s", dirname, tmp);
        printf("Probeparsing: %s\n", tmp2);
        print_s(tmp2);
    }

    closedir(d);



}

static void print_s(char *name)
{
    service_h *service = NULL;

    /* clear service cache */
    INIT_LIST_HEAD(&g.service_cache.list);

    service = parse_service(name);
    if (!service)
    {
        /* unload all modules */
        initng_unload_all_modules();
        initng_free();
        exit(2);
    }

    service_db_print_u(service, stdout, 1);

    /* this clears and fres all */
    service_db_free_all();
    INIT_LIST_HEAD(&g.service_cache.list);
}
